{***************************************************************************
                      AnalogInstrument.pas  -  description
                             -------------------
    begin             : 03.12.2012
    copyright         : (C) 2012 by MHS-Elektronik GmbH & Co. KG, Germany
    autho             : Klaus Demlehner, klaus@mhs-elektronik.de
 ***************************************************************************}

{***************************************************************************
 *                                                                         *
 *   This program is free software, you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License           *
 *   version 2.1 as published by the Free Software Foundation.             *
 *                                                                         *
 ***************************************************************************}
unit AnalogInstrument;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, EmpfangPrototyp, Hauptfenster,
  IntegerTerm, AnalogKonfig, Menus, A3nalogGauge;

type
  TAnalogInstrumentForm = class(TEmpfangPrototypForm)
    AnalogInstMenu: TPopupMenu;
    ConfigBtn: TMenuItem;
    AktivBtn: TMenuItem;
    N1: TMenuItem;
    DestroyBtn: TMenuItem;
    Gauge: TA3nalogGauge;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure ConfigBtnClick(Sender: TObject);
    procedure AktivBtnClick(Sender: TObject);
    procedure DestroyBtnClick(Sender: TObject);
  private
    { Private-Deklarationen }
    WidgetAktiv: boolean;
    BerechnungsObj: TIntTerm;
    Varis: VarArray;
    FSizeType: Integer;
    NachrichtenID: longword;
    BerechnungsTerm: string;
    procedure SetFSizeType(value: Integer);
    property SizeType: Integer read FSizeType write SetFSizeType;
  public
    { Public-Deklarationen }
    procedure NeueNachricht(CANrec: PCANrecord; var FreeMsg: boolean); override;
    procedure KonfigLaden(WerteListe: TStrings); override;
    procedure KonfigSichern(WerteListe: TStrings); override;
  end;

implementation

{$R *.dfm}

procedure TAnalogInstrumentForm.FormCreate(Sender: TObject);
var
  i: integer;
  MenuHandle: HMENU;

begin
inherited;
BerechnungsObj := TIntTerm.Create;
MenuHandle:=GetSystemMenu(Self.Handle, False);
DeleteMenu(MenuHandle, 7, MF_BYPOSITION);
DeleteMenu(MenuHandle, 6, MF_BYPOSITION);
DeleteMenu(MenuHandle, 4, MF_BYPOSITION);
SetLength(Varis, 8);
for i := 0 to high(Varis) do
  begin
  Varis[i].Name := format('d%u',[i]);
  Varis[i].Wert := 0;
  end;
//slef.Selected;
Gauge.FaceOptions := Gauge.FaceOptions - [ShowIndicatorMin, ShowIndicatorMid, ShowIndicatorMax];
WidgetAktiv := True;
SizeType := 1;
Gauge.ScaleMin := 0;
Gauge.ScaleMax := 100;
Gauge.NumberMainTicks := 10;
Gauge.NumberSubTicks := 5;
Gauge.IndMinimum := 30;
Gauge.IndMaximum := 70;
Gauge.Caption := 'x';
NachrichtenID := 0;
BerechnungsTerm := '(d0 << 8) + d1';
FormResize(self);
end;


procedure TAnalogInstrumentForm.FormResize(Sender: TObject);
begin
inherited;
self.ClientWidth := Gauge.Width;
self.ClientHeight := Gauge.Top + Gauge.Height;
end;

procedure TAnalogInstrumentForm.SetFSizeType(value: Integer);
begin
if value <> FSizeType then
  begin
  FSizeType := value;
  case value of
    0 : begin;
        Gauge.Width := 150;
        Gauge.Height := 150;
        Gauge.ArrowWidth := 1;
        Gauge.TicksWidth := 1;
        Gauge.Font.Size := 8;
        end;
    1 : begin;
        Gauge.Width := 250;
        Gauge.Height := 250;
        Gauge.ArrowWidth := 2;
        Gauge.TicksWidth := 2;
        Gauge.Font.Size := 8;
        end;
    2 : begin;
        Gauge.Width := 400;
        Gauge.Height := 400;
        Gauge.ArrowWidth := 4;
        Gauge.TicksWidth := 4;
        Gauge.Font.Size := 14;
        end;
    else
      Gauge.Width := 250;
      Gauge.Height := 250;
      Gauge.ArrowWidth := 2;
      Gauge.TicksWidth := 2;
      Gauge.Font.Size := 8;
    end;
  end;
end;


procedure TAnalogInstrumentForm.NeueNachricht(CANrec: PCANrecord; var FreeMsg: boolean);
var
  i: integer;
begin
if (not WidgetAktiv) or (CANrec.ID <> NachrichtenID) then
  exit;

for i := 0 to high(Varis) do
  Varis[i].Wert := CANrec.Data[i];

try
  Gauge.Position := BerechnungsObj.TermLoesen(BerechnungsTerm, @Varis);
  //self.Paint;
except
  WidgetAktiv := false;
  MessageDlg('Im Berechnungs Term ist ein Fehler aufgetretten:'+#13+#10+
        BerechnungsObj.GetFehlerText, mtError, [mbOk], 0);
end;
end;


procedure TAnalogInstrumentForm.KonfigLaden(WerteListe: TStrings);

begin
self.Caption := ReadListString(WerteListe, 'Name', self.Caption);
NachrichtenID := ReadListInteger(WerteListe, 'NachrichtenID', NachrichtenID);
Gauge.Caption := ReadListString(WerteListe, 'Einheit', Gauge.Caption);
BerechnungsTerm := ReadListString(WerteListe, 'Term', BerechnungsTerm);
Gauge.FaceColor := TColor(ReadListInteger(WerteListe, 'WidgetColor', Integer(Gauge.FaceColor)));
SizeType := ReadListInteger(WerteListe, 'SizeType', SizeType);
Gauge.ScaleMax := ReadListInteger(WerteListe, 'MaxWert', Gauge.ScaleMax);  // ScaleMin / Max getauscht <*>
Gauge.ScaleMin := ReadListInteger(WerteListe, 'MinWert', Gauge.ScaleMin);

Gauge.NumberMainTicks := ReadListInteger(WerteListe, 'SkalenTeilung', Gauge.NumberMainTicks);
Gauge.NumberSubTicks := ReadListInteger(WerteListe, 'SkalenSubTeilung', Gauge.NumberSubTicks);
Gauge.IndMinimum := ReadListInteger(WerteListe, 'IndMinimum', Gauge.IndMinimum);
Gauge.IndMaximum := ReadListInteger(WerteListe, 'IndMaximum', Gauge.IndMaximum);
if ReadListInteger(WerteListe, 'EnableMinRange', 0) > 0 then
  Gauge.FaceOptions := Gauge.FaceOptions + [ShowIndicatorMin]
else
  Gauge.FaceOptions := Gauge.FaceOptions - [ShowIndicatorMin];
if ReadListInteger(WerteListe, 'EnableMidRange', 0) > 0 then
  Gauge.FaceOptions := Gauge.FaceOptions + [ShowIndicatorMid]
else
  Gauge.FaceOptions := Gauge.FaceOptions - [ShowIndicatorMid];
if ReadListInteger(WerteListe, 'EnableMaxRange', 0) > 0 then
  Gauge.FaceOptions := Gauge.FaceOptions + [ShowIndicatorMax]
else
  Gauge.FaceOptions := Gauge.FaceOptions - [ShowIndicatorMax];
Gauge.MinColor := TColor(ReadListInteger(WerteListe, 'MinRangeColor', Integer(Gauge.MinColor)));
Gauge.MidColor := TColor(ReadListInteger(WerteListe, 'MidRangeColor', Integer(Gauge.MinColor)));
Gauge.MaxColor := TColor(ReadListInteger(WerteListe, 'MaxRangeColor', Integer(Gauge.MinColor)));  
FormResize(self);
FensterMenuItem.Caption :=  self.Caption;
end;


procedure TAnalogInstrumentForm.KonfigSichern(WerteListe: TStrings);

begin
WerteListe.Append(format('Name=%s', [self.Caption]));
WerteListe.Append(format('NachrichtenID=%u', [NachrichtenID]));
WerteListe.Append(format('Einheit=%s', [Gauge.Caption]));
WerteListe.Append(format('Term=%s', [BerechnungsTerm]));
WerteListe.Append(format('WidgetColor=%u', [Integer(Gauge.FaceColor)]));
WerteListe.Append(format('SizeType=%u', [SizeType]));
WerteListe.Append(format('MinWert=%d', [Gauge.ScaleMin]));
WerteListe.Append(format('MaxWert=%d', [Gauge.ScaleMax]));
WerteListe.Append(format('SkalenTeilung=%u', [Gauge.NumberMainTicks]));
WerteListe.Append(format('SkalenSubTeilung=%u', [Gauge.NumberSubTicks]));
WerteListe.Append(format('IndMinimum=%u', [Gauge.IndMinimum]));
WerteListe.Append(format('IndMaximum=%u', [Gauge.IndMaximum]));
if ShowIndicatorMin in Gauge.FaceOptions then
  WerteListe.Append('EnableMinRange=1')
else
  WerteListe.Append('EnableMinRange=0');
if ShowIndicatorMid in Gauge.FaceOptions then
  WerteListe.Append('EnableMidRange=1')
else
  WerteListe.Append('EnableMidRange=0');
if ShowIndicatorMax in Gauge.FaceOptions then
  WerteListe.Append('EnableMaxRange=1')
else
  WerteListe.Append('EnableMaxRange=0');
WerteListe.Append(format('MinRangeColor=%u', [Integer(Gauge.MinColor)]));
WerteListe.Append(format('MidRangeColor=%u', [Integer(Gauge.MidColor)]));
WerteListe.Append(format('MaxRangeColor=%u', [Integer(Gauge.MaxColor)]));
end;


procedure TAnalogInstrumentForm.ConfigBtnClick(Sender: TObject);
var
  Form: TAnalogKonfigForm;

begin
inherited;
Form := TAnalogKonfigForm.Create(self);

Form.NameEdit.Text := self.Caption;
Form.CANIDEdit.Text := TMainForm(Owner).GetIDString(NachrichtenID);
Form.EinheitEdit.Text := Gauge.Caption;
Form.BerechnungsTermEdit.Text := BerechnungsTerm;
Form.ColorEdit.Selected := Gauge.FaceColor;
Form.SizeEdit.ItemIndex := SizeType;
Form.MinSkalaSpin.Value := Gauge.ScaleMin;
Form.MaxSkalaSpin.Value := Gauge.ScaleMax;
Form.SkalaTeilungSpin.Value := Gauge.NumberMainTicks;
Form.SkalaUnterteilungSpin.Value := Gauge.NumberSubTicks;
Form.IndMinSpin.Value := Gauge.IndMinimum;
Form.IndMaxSpin.Value := Gauge.IndMaximum;
if ShowIndicatorMin in Gauge.FaceOptions then
  Form.MinRangeCheck.Checked := True
else
  Form.MinRangeCheck.Checked := False;
if ShowIndicatorMid in Gauge.FaceOptions then
  Form.MidRangeCheck.Checked := True
else
  Form.MidRangeCheck.Checked := False;
if ShowIndicatorMax in Gauge.FaceOptions then
  Form.MaxRangeCheck.Checked := True
else
  Form.MaxRangeCheck.Checked := False;
Form.MinRangeColorEdit.Selected := Gauge.MinColor; 
Form.MidRangeColorEdit.Selected := Gauge.MidColor;
Form.MaxRangeColorEdit.Selected := Gauge.MaxColor;

if Form.ShowModal = mrOK then
  begin
  self.Caption := Form.NameEdit.Text;
  NachrichtenID := Form.CANIDEdit.Tag;
  Gauge.Caption := Form.EinheitEdit.Text;
  BerechnungsTerm := Form.BerechnungsTermEdit.Text;
  Gauge.FaceColor := Form.ColorEdit.Selected;
  SizeType := Form.SizeEdit.ItemIndex;
  Gauge.ScaleMin := Form.MinSkalaSpin.Value;
  Gauge.ScaleMax := Form.MaxSkalaSpin.Value;
  Gauge.NumberMainTicks := Form.SkalaTeilungSpin.Value;
  Gauge.NumberSubTicks := Form.SkalaUnterteilungSpin.Value;
  Gauge.IndMinimum := Form.IndMinSpin.Value;
  Gauge.IndMaximum := Form.IndMaxSpin.Value;
  if Form.MinRangeCheck.Checked then
    Gauge.FaceOptions := Gauge.FaceOptions + [ShowIndicatorMin]
  else
    Gauge.FaceOptions := Gauge.FaceOptions - [ShowIndicatorMin];
  if Form.MidRangeCheck.Checked then
    Gauge.FaceOptions := Gauge.FaceOptions + [ShowIndicatorMid]
  else
    Gauge.FaceOptions := Gauge.FaceOptions - [ShowIndicatorMid];
  if Form.MaxRangeCheck.Checked then
    Gauge.FaceOptions := Gauge.FaceOptions + [ShowIndicatorMax]
  else
    Gauge.FaceOptions := Gauge.FaceOptions - [ShowIndicatorMax];
  Gauge.MinColor := Form.MinRangeColorEdit.Selected;
  Gauge.MidColor := Form.MidRangeColorEdit.Selected;
  Gauge.MaxColor := Form.MaxRangeColorEdit.Selected;  
  FormResize(self);
  FensterMenuItem.Caption :=  self.Caption;
  end;
Form.Free;
end;


procedure TAnalogInstrumentForm.AktivBtnClick(Sender: TObject);

begin
inherited;
WidgetAktiv := AktivBtn.Checked;
end;


procedure TAnalogInstrumentForm.DestroyBtnClick(Sender: TObject);

begin
inherited;
close;
end;

end.
