{***************************************************************************
                      BitIndikator.pas  -  description
                             -------------------
    begin             : 03.12.2012
    copyright         : (C) 2012 by MHS-Elektronik GmbH & Co. KG, Germany
    autho             : Klaus Demlehner, klaus@mhs-elektronik.de
 ***************************************************************************}

{***************************************************************************
 *                                                                         *
 *   This program is free software, you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License           *
 *   version 2.1 as published by the Free Software Foundation.             *
 *                                                                         *
 ***************************************************************************}
unit BitIndikator;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, EmpfangPrototyp, StdCtrls, Buttons, Imglist, StrUtils, Hauptfenster,
  IntegerTerm, ExtCtrls, Menus, Contnrs, Kr_Led;

type
  TBitConf = record
    Name: String[40];
    Color: TColor;
    BytePos: Byte;
    BitPos: Byte;
    end;

  PBitConf = ^TBitConf;

  TBitIndikatorForm = class(TEmpfangPrototypForm)
    BitIndikMenu: TPopupMenu;
    ConfigBtn: TMenuItem;
    AktivBtn: TMenuItem;
    N1: TMenuItem;
    DestroyBtn: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ConfigBtnClick(Sender: TObject);
    procedure AktivBtnClick(Sender: TObject);
    procedure DestroyBtnClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private-Deklarationen }
    WidgetAktiv: boolean;
    NachrichtenID: longword;  
    BitConfListe: TList;
    LEDS: TObjectList;   
    procedure CreateLEDS;
  public
    { Public-Deklarationen }
    procedure BitConfListeClear(liste: TList);
    procedure BitConfListeAdd(liste: TList; Name: String;
      LedColor: TColor; BytePos, BitPos: Byte);
    procedure NeueNachricht(CANrec: PCANrecord; var FreeMsg: boolean); override;
    procedure KonfigLaden(WerteListe: TStrings); override;
    procedure KonfigSichern(WerteListe: TStrings); override;
  end;

var
  BitIndikatorForm: TBitIndikatorForm;

implementation

uses BitIndikatorKonfig;

{$R *.dfm}

// <*> gehört hier nicht rein
function ExtractSubstr(const S: string; var Pos: Integer;
  const Delims: TSysCharSet): string;
var
  I: Integer;
begin
I := Pos;
while (I <= Length(S)) and not (S[I] in Delims) do Inc(I);
Result := Copy(S, Pos, I - Pos);
if (I <= Length(S)) and (S[I] in Delims) then Inc(I);
Pos := I;
end;


procedure TBitIndikatorForm.FormCreate(Sender: TObject);
var
  MenuHandle: HMENU;
  
begin
inherited;
MenuHandle:=GetSystemMenu(Self.Handle, False);
DeleteMenu(MenuHandle, 7, MF_BYPOSITION);
DeleteMenu(MenuHandle, 6, MF_BYPOSITION);
DeleteMenu(MenuHandle, 4, MF_BYPOSITION);

BitConfListe := TList.Create;
LEDS := TObjectList.Create;

WidgetAktiv := True;
NachrichtenID := 0;
end;


procedure TBitIndikatorForm.FormDestroy(Sender: TObject);

begin
inherited;
LEDS.Free;
BitConfListeClear(BitConfListe);
BitConfListe.Free;
end;


procedure TBitIndikatorForm.BitConfListeClear(liste: TList);
var i: Integer;

begin;
if liste.Count = 0 then
  exit;
for i := 0 to liste.Count-1 do
  dispose(liste.Items[i]);
liste.Clear;
end;


procedure TBitIndikatorForm.BitConfListeAdd(liste: TList; Name: String;
    LedColor: TColor; BytePos, BitPos: Byte);
var item: PBitConf;
    
begin
new(item);
item.Name := Name;       // Bezeichnung
item.Color := LedColor;  // Farbe
item.BytePos := BytePos; // Byte
item.BitPos := BitPos;   // Bit
liste.Add(item);
end;


procedure TBitIndikatorForm.FormResize(Sender: TObject);
var
  i, h, top: integer;

begin
inherited;
if LEDS.Count = 0 then
  exit;
h := KrLed(LEDS[0]).Height;
top := 0;  
for i := 0 to LEDS.Count-1 do
  begin
  KrLed(LEDS[i]).Top := top;
  top := top + h;
  end;
self.ClientWidth := KrLed(LEDS[0]).Width;   
self.ClientHeight := top;
end;  


procedure TBitIndikatorForm.KonfigLaden(WerteListe: TStrings);
const
  Delims = ['=', ';'];
var
  i, Pos, Idx: integer;
  Str, Item, Name: String;
  LedColor: TColor;
  BytePos, BitPos: Byte;

begin
self.Caption := ReadListString(WerteListe, 'Name', self.Caption);
NachrichtenID := ReadListInteger(WerteListe, 'NachrichtenID', NachrichtenID);
BitConfListeClear(BitConfListe);
for i := 0 to WerteListe.Count-1 do
  begin
  Str := WerteListe.Strings[i];
  if AnsiStartsText('LED', Str) then
    begin
    Pos := 1;
    Idx := 0;
    LedColor := clRed;
    BytePos := 0;
    BitPos := 0;
    while Pos <= Length(Str) do
      begin
      Item := ExtractSubstr(Str, Pos, Delims);
      case Idx of
        1: Name := Item;
        2: LedColor := StringToColor(Item);
        3: BytePos := StrToInt(Item);
        4: BitPos := StrToInt(Item);
        end;
      Inc(Idx);
      end;
    BitConfListeAdd(BitConfListe, Name, LedColor, BytePos, BitPos);
    end;
  end;
CreateLEDS;
FensterMenuItem.Caption :=  self.Caption;
end;


procedure TBitIndikatorForm.KonfigSichern(WerteListe: TStrings);
var
  i: integer;
  BytePos, BitPos : Byte;
  Name, LedColor: String;
  
begin
WerteListe.Append(format('Name=%s', [self.Caption]));
WerteListe.Append(format('NachrichtenID=%u', [NachrichtenID]));
if BitConfListe.Count > 0 then
  begin;
  for i := 0 to BitConfListe.Count-1 do
    begin;
    Name := PBitConf(BitConfListe[i]).Name; 
    LedColor := ColorToString(PBitConf(BitConfListe[i]).Color); 
    BytePos := PBitConf(BitConfListe[i]).BytePos;
    BitPos := PBitConf(BitConfListe[i]).BitPos;
    WerteListe.Append(format('LED%u=%s;%s;%u;%u', [i, Name, LedColor, BytePos, BitPos]));      
    end;
  end;  
end;


procedure TBitIndikatorForm.CreateLEDS;
var i: Integer;
    led: KrLed;

begin;
LEDS.Clear;
if BitConfListe.Count = 0 then
  exit;
for i := 0 to BitConfListe.Count-1 do
  begin
  led := KrLed.Create(self);
  led.Parent := self;
  led.Left := 0;
  led.Height := 25;
  led.LedCircle := 16;
  led.Width := 200;
  led.LedOffColor := clSilver;
  led.Font.Style := [fsBold];
  led.LedOnColor := PBitConf(BitConfListe[i]).Color;
  led.Caption := PBitConf(BitConfListe[i]).Name;
  led.LedState := stOff;
  LEDS.Add(led);
  end;
FormResize(self);
end;


procedure TBitIndikatorForm.NeueNachricht(CANrec: PCANrecord; var FreeMsg: boolean);
var
  i: integer;
  maske: Byte;
  
begin
if (not WidgetAktiv) or (CANrec.ID <> NachrichtenID) then
  exit;

if BitConfListe.Count = 0 then
  exit;
for i := 0 to BitConfListe.Count-1 do
  begin
  if i >= LEDS.Count then
    exit;
  maske := 1 shl PBitConf(BitConfListe[i]).BitPos;  
  if (CANrec.Data[PBitConf(BitConfListe[i]).BytePos] and maske) > 0 then
    KrLed(LEDS.Items[i]).LedState := stOn
  else
    KrLed(LEDS.Items[i]).LedState := stOff;
  end;
end;


procedure TBitIndikatorForm.ConfigBtnClick(Sender: TObject);
var
  Form: TBitKonfigForm;
  
begin
inherited;
Form := TBitKonfigForm.Create(self);

Form.NameEdit.Text := self.Caption;
Form.CANIDEdit.Text := TMainForm(Owner).GetIDString(NachrichtenID);
Form.SetBitConfListe(BitConfListe);
if Form.ShowModal = mrOK then
  begin
  self.Caption := Form.NameEdit.Text;
  NachrichtenID := Form.CANIDEdit.Tag;
  Form.GetBitConfListe(BitConfListe);
  CreateLEDS;
  FensterMenuItem.Caption :=  self.Caption;
  end;
Form.Free;
end;


procedure TBitIndikatorForm.AktivBtnClick(Sender: TObject);

begin
inherited;
WidgetAktiv := AktivBtn.Checked;
end;


procedure TBitIndikatorForm.DestroyBtnClick(Sender: TObject);
begin
  inherited;
close;
end;

end.
