{***************************************************************************
                      WertAnzeige.pas  -  description
                             -------------------
    begin             : 03.12.2012
    copyright         : (C) 2012 by MHS-Elektronik GmbH & Co. KG, Germany
    autho             : Klaus Demlehner, klaus@mhs-elektronik.de
 ***************************************************************************}

{***************************************************************************
 *                                                                         *
 *   This program is free software, you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License           *
 *   version 2.1 as published by the Free Software Foundation.             *
 *                                                                         *
 ***************************************************************************}
unit WertAnzeige;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, EmpfangPrototyp, StdCtrls, Buttons, Hauptfenster, IntegerTerm,
  ExtCtrls, Menus, RackCtls;

type
  TWertAnzeigeForm = class(TEmpfangPrototypForm)
    EinheitLabel: TLabel;
    WertAnzMenu: TPopupMenu;
    ConfigBtn: TMenuItem;
    AktivBtn: TMenuItem;
    N1: TMenuItem;
    DestroyBtn: TMenuItem;
    LEDDisplay: TLEDDisplay;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure ConfigBtnClick(Sender: TObject);
    procedure AktivBtnClick(Sender: TObject);
    procedure DestroyBtnClick(Sender: TObject);
  private
    { Private-Deklarationen }
    WidgetAktiv: boolean;
    BerechnungsObj: TIntTerm;
    Varis: VarArray;
    NachrichtenID: longword;
    BerechnungsTerm: string;
    Einheit: string;
    Margin: integer;       // Abstand der Skala zum Rand
    FSizeType: Integer;
    procedure SetFSizeType(value: Integer);
    property SizeType: Integer read FSizeType write SetFSizeType;
  public
    { Public-Deklarationen }
    procedure NeueNachricht(CANrec: PCANrecord; var FreeMsg: boolean); override;
    procedure KonfigLaden(WerteListe: TStrings); override;
    procedure KonfigSichern(WerteListe: TStrings); override;
  end;

var
  WertAnzeigeForm: TWertAnzeigeForm;

implementation

uses WerteKonfig;

{$R *.dfm}

procedure TWertAnzeigeForm.FormCreate(Sender: TObject);
var
  i: integer;
  MenuHandle: HMENU;

begin
inherited;
BerechnungsObj := TIntTerm.Create;
MenuHandle:=GetSystemMenu(Self.Handle, False);
DeleteMenu(MenuHandle, 7, MF_BYPOSITION);
DeleteMenu(MenuHandle, 6, MF_BYPOSITION);
DeleteMenu(MenuHandle, 4, MF_BYPOSITION);

SetLength(Varis, 8);
for i := 0 to high(Varis) do
  begin
  Varis[i].Name := format('d%u',[i]);
  Varis[i].Wert := 0;
  end;

WidgetAktiv := True;
Margin := 5;
Einheit := 'x';
NachrichtenID := 0;
BerechnungsTerm := '(d0 << 8) + d1';
SizeType := 1;
LEDDisplay.FractionDigits := 0;
LEDDisplay.Value := 0;
FormResize(self);
end;


procedure TWertAnzeigeForm.FormResize(Sender: TObject);
var h: Integer;

begin
inherited;
LEDDisplay.Left := Margin;
EinheitLabel.Left := LEDDisplay.Left + LEDDisplay.Width + Margin;
EinheitLabel.Width := EinheitLabel.Canvas.TextWidth(Einheit);
EinheitLabel.Height := EinheitLabel.Canvas.TextHeight(Einheit);
self.ClientWidth := LEDDisplay.Width + EinheitLabel.Width + (2 * Margin);
h := LEDDisplay.Top + LEDDisplay.Height + Margin;
EinheitLabel.Top := h - (EinheitLabel.Height + 5);
self.ClientHeight := h;
EinheitLabel.Caption := Einheit;
end;


procedure TWertAnzeigeForm.SetFSizeType(value: Integer);
begin
if value <> FSizeType then
  begin
  FSizeType := value;
  case value of
    0 : begin;
        EinheitLabel.Font.Size := 10;
        LEDDisplay.DigitLineWidth := 2;
        LEDDisplay.DigitHeight := 28;
        LEDDisplay.DigitWidth := 15;
        LEDDisplay.Height := 30;
        LEDDisplay.Width := 120;
        end;
    1 : begin;
        EinheitLabel.Font.Size := 18;
        LEDDisplay.DigitLineWidth := 4;
        LEDDisplay.DigitHeight := 50;
        LEDDisplay.DigitWidth := 28;
        LEDDisplay.Height := 57;
        LEDDisplay.Width := 209;
        end;
    2 : begin;
        EinheitLabel.Font.Size := 28;
        LEDDisplay.DigitLineWidth := 7;
        LEDDisplay.DigitHeight := 90;
        LEDDisplay.DigitWidth := 57;
        LEDDisplay.Height := 100;
        LEDDisplay.Width := 400;
        end;
    else

    end;
  end;
end;


procedure TWertAnzeigeForm.KonfigLaden(WerteListe: TStrings);

begin
self.Caption := ReadListString(WerteListe, 'Name', self.Caption);
NachrichtenID := ReadListInteger(WerteListe, 'NachrichtenID', NachrichtenID);
BerechnungsTerm := ReadListString(WerteListe, 'Term', BerechnungsTerm);
Einheit := ReadListString(WerteListe, 'Einheit', Einheit);
LEDDisplay.ColorLED := TColor(ReadListInteger(WerteListe, 'WidgetColor', Integer(LEDDisplay.ColorLED)));
EinheitLabel.Font.Color := LEDDisplay.ColorLED;
SizeType := ReadListInteger(WerteListe, 'SizeType', SizeType);
LEDDisplay.FractionDigits := ReadListInteger(WerteListe, 'PrecDigits', LEDDisplay.FractionDigits);
FormResize(self);
FensterMenuItem.Caption :=  self.Caption;
end;


procedure TWertAnzeigeForm.KonfigSichern(WerteListe: TStrings);

begin
WerteListe.Append(format('Name=%s', [self.Caption]));
WerteListe.Append(format('NachrichtenID=%u', [NachrichtenID]));
WerteListe.Append(format('Term=%s', [BerechnungsTerm]));
WerteListe.Append(format('Einheit=%s', [Einheit]));
WerteListe.Append(format('WidgetColor=%u', [Integer(LEDDisplay.ColorLED)]));
WerteListe.Append(format('SizeType=%u', [SizeType]));
WerteListe.Append(format('PrecDigits=%d', [LEDDisplay.FractionDigits]));
end;


procedure TWertAnzeigeForm.NeueNachricht(CANrec: PCANrecord; var FreeMsg: boolean);
var
  i: integer;

begin
if (not WidgetAktiv) or (CANrec.ID <> NachrichtenID) then
  exit;

for i := 0 to high(Varis) do
  Varis[i].Wert := CANrec.Data[i];

try
  LEDDisplay.Value := BerechnungsObj.TermLoesen(BerechnungsTerm, @Varis);
except
  WidgetAktiv := false;
  MessageDlg('Im Berechnungs Term ist ein Fehler aufgetretten:'+#13+#10+
        BerechnungsObj.GetFehlerText, mtError, [mbOk], 0);
  end;
end;


procedure TWertAnzeigeForm.ConfigBtnClick(Sender: TObject);
var
  Form: TWerteKonfigForm;

begin
inherited;
Form := TWerteKonfigForm.Create(self);

Form.NameEdit.Text := self.Caption;
Form.CANIDEdit.Text := TMainForm(Owner).GetIDString(NachrichtenID);
Form.BerechnungsTermEdit.Text := BerechnungsTerm;
Form.EinheitEdit.Text := Einheit;
Form.ColorEdit.Selected := LEDDisplay.ColorLED;
Form.SizeEdit.ItemIndex := SizeType;
Form.PrecDigitEdit.Value := LEDDisplay.FractionDigits;

if Form.ShowModal = mrOK then
  begin
  self.Caption := Form.NameEdit.Text;
  NachrichtenID := Form.CANIDEdit.Tag;
  BerechnungsTerm := Form.BerechnungsTermEdit.Text;
  Einheit := Form.EinheitEdit.Text;
  LEDDisplay.ColorLED := Form.ColorEdit.Selected;
  EinheitLabel.Font.Color := LEDDisplay.ColorLED;
  SizeType := Form.SizeEdit.ItemIndex;
  LEDDisplay.FractionDigits := Form.PrecDigitEdit.Value;

  FormResize(self);
  FensterMenuItem.Caption :=  self.Caption;
  end;
Form.Free;
end;


procedure TWertAnzeigeForm.AktivBtnClick(Sender: TObject);
begin
inherited;
WidgetAktiv := AktivBtn.Checked;
end;


procedure TWertAnzeigeForm.DestroyBtnClick(Sender: TObject);
begin
inherited;
close;
end;

end.
